Unit ALN1R1_p;
interface
uses
SysUtils, Math, { Delphi }
Lstruct, Lfunc;

procedure ALN1R1(MODE :Integer; LPIVOT :Integer; L1 :Integer;
                M :Integer; var U :Array of Real; IUE :Integer;
                var UP :Real; var C :Array of Real; ICE :Integer;
                ICV :Integer; NCV :Integer);

implementation

procedure ALN1R1(MODE :Integer; LPIVOT :Integer; L1 :Integer;
                M :Integer; var U :Array of Real; IUE :Integer;
                var UP :Real; var C :Array of Real; ICE :Integer;
                ICV :Integer; NCV :Integer);
var
I,INCR,I2,I3,I4,J :Integer;
CL,CLINV,SM1 :Real;
B,SM :Extended;
const
DONE :Real = 1.0e0;
ONE :Real = 1.0E0;
SYS056 :Real = 1.175495E-38;
label
_10,_20,_30,_40,_50,_60,_70,_80,_90,_100,_110,_120,_130,_140,_150,
_160,_170;
begin

{ ********************************************************* }

{     PURPOSE: }
{        CONSTRUCTION AND/OR APPLICATION OF A SINGLE }
{        HOUSEHOLEER TRANSFORMATION }

{     REFERENCES: }
{    1. GOLUB G.H., AND VAN LOAN CH.F. MATRIX }
{       COMPUTATIONS.-JOHNS HOPKINS UNIV.PRESS, 1983 }
{    2. LAWSON C.L., AND HANSON R.J. SOLVING LEAST SQUARES }
{       PROBLEMS. - PRENTICE-HALL,INC.,ENGLEWOOE CLIFFS, }
{       NEW JERSEY, 1974 }

{     USAGE: }
{    CALL ALN1R1 (MODE,LPIVOT,L1,M,U,IUE,UP,C,ICE,ICV,NCV) }

{     PARAMETERS: }

{     MODE - INTEGER VARIABLE - A CONTROL PARAMETER TO }
{            SELECT ALGORITHM H1 (CONSTRUCTION AND APPLI- }
{            CATION OF Q) OR H2 (APPLICATION ONLY) [2]; }
{   LPIVOT - INTEGER VARIABLE - IS THE INDEX OF PIVOT }
{            ENTRY LP ; }
{       L1 - INTEGER VARIABLE - IS DESCRIBEE ABOVE; }
{            IF L1 <= M, THE TRANSFORMATION WILL BE }
{            CONSTRUCTEE TO ZERO ELEMENTS INDEXED FROM L1 }
{            THROUGH M. IF L1 > M  THE SUBROUTINE DOES }
{            AN IDENTITY TRANSFORMATION; }
{       M - INTEGER VARIABLE - THE SIZE OF PROBLEM (THE }
{           LENGTH OF VECTOR V DESCRIBEE ABOVE); }
{       U - REAL ARRAY OF SIZE IUE BY M - ON ENTRANCE }
{           TO H1 CONTAINS THE PIVOT VECTOR V WITH THE }
{           STORAGE INCREMENT IUE BETWEEN ELEMENTS; }
{           ON EXIT FROM H1 U AND UP CONTAIN QUANTITIES }
{           DEFINING THE VECTOR U OF THE HOUSEHOLEER }
{           TRANSFORMATION (SEE (4)); }
{           ON ENTRY TO H2 U AND UP SHOULE CONTAIN QUANTI- }
{           TIES PREVIOUSLY COMPUTEE BY H1. THESE WILL NOT }
{           BE MODIFIEE BY H2; }
{    IUE - INTEGER VARIABLE - IS THE STORAGE INCREMENT }
{          BETWEEN ELEMENTS OF U; }
{     UP - REAL VARIABLE - ON ENTRANCE TO H1 IS ARBITRARY; }
{          ON EXIT FROM H1 CONTAINS U(LP) (SEE (4)); }
{          ON ENTRANCE TO H2 UP SHOULE CONTAIN U(LP) PRE- }
{          VIOUSLY COMPUTEE BY H1 AND WILL NOT CHANGE BY }
{          H2; }
{      C - REAL VECTOR OF LENGTH  ICV*NCV  AT LEAST IF }
{          EL = Q*F MUST BE CALCULATEE AND OF LENGTH }
{          ICE*M  AT LEAST IF  ER = F*Q  MUST BE CALCU- }
{          LATEE  -  ON ENTRY TO H1 OR H2  C  CONTAINS A }
{          MATRIX WHICH BE REGAREED AS A SET OF VECTORS }
{          TO WHICH THE HOUSEHOLEER TRANSFORMATION IS TO }
{          BE APPLIED. ON EXIT  C  CONTAINS THE SET OF }
{          TRANSFORMEE VECTORS; }
{    ICE - INTEGER VARIABLE - STORAGE INCREMENT BETWEEN }
{          ELEMENTS OF VECTORS IN C; }
{    ICV - INTEGER VARIABLE - STORAGE INCREMENT BETWEEN }
{          VECTORS IN  C; }
{    NCV - INTEGER VARIABLE - NUMBER OF VECTORS IN  C  TO }
{          BE TRANSFORMED. IF  NCV <= 0 NO OPERATIONS }
{          WILL BE DONE ON  C . }

{ ********************************************************* }

{      SYS056 = SYS055 }

{        SYS056 IS INVERSE VALUE OF MAXIMAL REAL TYPE }
{        NUMBER WHICH MAY BE REPRESENTEE AT THE IBM PC . }

if ( 0 >= LPIVOT ) or ( LPIVOT >= L1 ) or ( L1 > M ) 
 then goto _170;
CL := Abs(U[0+(LPIVOT-1)*IUE]);
if ( MODE = 2 ) 
 then goto _100;

{      .....CONSTRUCT THE TRANSFORMATION......... }

for J:=L1 to M do
 begin
_10:
  CL := EMax1(Abs(U[0+(J-1)*IUE]),CL);
 end;
if ( CL ) < 0
 then goto _170
 else if ( CL ) > 0
       then goto _20
       else goto _170;
_20:
if ( CL-SYS056 ) < 0
 then goto _30
 else if ( CL-SYS056 ) > 0
       then goto _50
       else goto _30;
_30:
SM := IntPower((U[0+(LPIVOT-1)*IUE])/CL,2);
for J:=L1 to M do
 begin
_40:
  SM := SM+IntPower((U[0+(J-1)*IUE])/CL,2);
 end;
goto _70;
_50:
CLINV := ONE/CL;
SM := IntPower((U[0+(LPIVOT-1)*IUE])*CLINV,2);
for J:=L1 to M do
 begin
_60:
  SM := SM+IntPower((U[0+(J-1)*IUE])*CLINV,2);
 end;

{      CONVERT Extended PRECISION .SM. }
{      TO SINGLE PRECISION .SM1. }

_70:
SM1 := (Sqrt(SM));
CL := CL*SM1;
if ( U[0+(LPIVOT-1)*IUE] ) < 0
 then goto _90
 else if ( U[0+(LPIVOT-1)*IUE] ) > 0
       then goto _80
       else goto _90;
_80:
CL := -CL;
_90:
UP := U[0+(LPIVOT-1)*IUE]-CL;
U[0+(LPIVOT-1)*IUE] := CL;
goto _110;

{      APPLY THE TRANSFORMATION  1+U*TRANSPOSE(U)/B TO C* }

_100:
if ( CL ) < 0
 then goto _170
 else if ( CL ) > 0
       then goto _110
       else goto _170;
_110:
if ( NCV <= 0 ) 
 then goto _170;
B := (UP)*U[0+(LPIVOT-1)*IUE];

{      B MUST BE NONPOSITIVE HERE. IF B=0. RETURN }

if ( B ) < 0
 then goto _120
 else if ( B ) > 0
       then goto _170
       else goto _170;
_120:
B := DONE/B;
I2 := 1-ICV+ICE*(LPIVOT-1);
INCR := ICE*(L1-LPIVOT);
for J:=1 to NCV do
 begin
  I2 := I2+ICV;
  I3 := I2+INCR;
  I4 := I3;
  SM := C[I2-1]*(UP);
  for I:=L1 to M do
   begin
    SM := SM+C[I3-1]*(U[0+(I-1)*IUE]);
_130:
    I3 := I3+ICE;
   end;
  if ( SM ) < 0
   then goto _140
   else if ( SM ) > 0
         then goto _140
         else goto _160;
_140:
  SM := SM*B;
  C[I2-1] := C[I2-1]+SM*(UP);
  for I:=L1 to M do
   begin
    C[I4-1] := C[I4-1]+SM*(U[0+(I-1)*IUE]);
_150:
    I4 := I4+ICE;
   end;
_160:
 end;
_170:
end;

end.
