Unit ALN1R2_p;
interface
uses
SysUtils, Math, { Delphi }
Lstruct, Lfunc;

procedure ALN1R2(A :Real; B :Real; var COS :Real; var SIN :Real;
                var SIG :Real);

implementation

procedure ALN1R2(A :Real; B :Real; var COS :Real; var SIN :Real;
                var SIG :Real);
var
XR,YR :Real;
const
ONE :Real = 1.0E0;
ZERO :Real = 0.0E0;
label
_10,_20,_30,_40;
begin

{ ********************************************************* }

{     PURPOSE: }
{     CALCULATION OF ORTHOGONAL ROTATON MATRIX OF GIVENS. }

{     METHOD: }
{        FOR GIVEN 2-VECTOR  ( A ) WE }
{                            ( B ) }

{        COMPUTE MATRIX  G = (  COS , SIN  ) SO THAT }
{                            ( -SIN , COS  ) }

{      (  COS     SIN ) ( A ) = ( SQRT(A**2+B**2) ) }
{      ( -SIN     COS ) ( B ) = ( 0               ). }

{    COMPUTE SIG = SQRT(A**2+B**2). }
{    SIG IS COMPUTEE LAST TO ALLOW FOR THE POSSIBILITY }
{    THAT SIG MAY BE IN THE SAME LOCATION AS A OR B. }
{    SUBROUTINE ALN1R2 IS MODIFICATION OF G1 FROM [1]. }

{     REFERENCES: }
{     1.LAWSON C.L. AND HANSON R.J. SOLVING LEAST SQUARES }
{     PROBLEMS.- PRENTICE-HALL,INC.,ENGLEWOOE CLIFFS,NEW }
{     JERSEY,1974. }

{    USAGE: }
{        CALL ALN1R2(A,B,COS,SIN,SIG) }

{      INPUT PARAMETERS: }
{   A - REAL VARIABLE- THE FIRST COMPONENT OF VECTOR; }
{   B - REAL VARIABLE - THE SECONE COMPONENT OF VECTOR }

{      OUTPUT PARAMETERS: }
{   COS -REAL VARIABLE-THE FIRST ELEMENT OF GIVENS MATRIX; }
{   SIN -REAL VARIABLE-THE SECONE ELEMENT OF GIVENS MATRIX }
{   SIG -REAL VARIABLE-THE EVALUATEE NORM OF 2-VECTOR }
{        (A,B). }

{      STANEARE FUNCTIONaln1r2 AND PROCEDUREaln1r2: ABS , SQRT . }

{ ********************************************************* }


{       SEARCH OF A MAXIMAL COMPONENT OF ( A , B ). }

if ( Abs(A) <=  Abs(B) ) 
 then goto _10;

{     A  IS A MAXIMAL COMPONENT. }
{     NORMALIZATION TO PREVENT THE OVERFLOW OR UNDERFLOW. }

XR := B/A;
YR := Sqrt(ONE+IntPower(XR,2));
COS := Sign(ONE/YR,A);
SIN := COS*XR;
SIG := Abs(A)*YR;
goto _40;

{        CHECK OF NONZERO VALUE OF  B . }

_10:
if ( B ) < 0
 then goto _20
 else if ( B ) > 0
       then goto _20
       else goto _30;

{     B  IS THE MAXIMAL COMPONENT. }
{     NORMALIZATION FOR PREVENT THE OVERFLOW OR UNDERFLOW. }

_20:
XR := A/B;
YR := Sqrt(ONE+IntPower(XR,2));
SIN := Sign(ONE/YR,B);
COS := SIN*XR;
SIG := Abs(B)*YR;
goto _40;

{         BOTH  A  AND  B  ARE ZERO'S. }

_30:
SIG := ZERO;
COS := ZERO;
SIN := ONE;

{        END OF CALCULATION OF GIVENS ROTATION MATRIX. }

_40:
end;

end.
