Unit ALN1R_p;
interface
uses
SysUtils, Math, { Delphi }
Lstruct, Lfunc, ALN1R1_p, ALN1R2_p, ALN1R3_p, ALN1R4_p;

procedure ALN1R(var A :Array of Real; MDA :Integer;
                M :Integer; N :Integer; var B :Array of Real;
                var X :Array of Real; var RNORM :Real;
                var W :Array of Real; var ZZ :Array of Real;
                var INDEX :Array of Integer; var IERR :Integer);

implementation

procedure ALN1R(var A :Array of Real; MDA :Integer;
                M :Integer; N :Integer; var B :Array of Real;
                var X :Array of Real; var RNORM :Real;
                var W :Array of Real; var ZZ :Array of Real;
                var INDEX :Array of Integer; var IERR :Integer);
var
I,II,IP,ITER,ITMAX,IZ,IZ1,IZ2,IZMAX,J,JJ,JZ,L,NEXT,NPP1,NSETP :Integer;
ALPHA,ASAVE,CC,DUMMY,SM,SS,T,UNORM,UP,WMAX,ZTEST :Real;
const
FACTOR :Real = 1.0E-02;
ONE :Real = 1.0E0;
TWO :Real = 2.0E0;
ZERO :Real = 0.0E0;
label
_10,_20,_30,_40,_50,_60,_70,_80,_90,_100,_110,_120,_130,_140,_150,
_160,_170,_180,_190,_200,_210,_220,_230,_240,_250,_260,_270,_280,_290,
_300,_310,_320,_330,_340,_350,_360,_370,_380,_390,_400,_410,_420,_430,
_450;
begin

{ ******************************************************* }

{     PURPOSE: *****NONNEGATIVE LEAST SQUARES*********** }

{   GIVEN AN M BY N  MATRIX  .A. AND  AN  M-VECTOR .B. }
{   COMPUTE  AN  N-VECTOR .X.  WHICH SOLVES THE LEAST }
{   SQUARES PROBLEM: }
{           A * X =  B     SUBJECT TO X.GE.0 }

{   SOURCE:LAWSON C.L.,HANSON R.J. 'SOLVING LEAST SQUARES }
{   PROBLEM' , PRENTICE-HALL INC.,ENGLEWOOE CLIFFS,NEW }
{   JERSEY, 1974. }

{     PARAMETERS: }
{  A,MDA,M,N - MDA IS THE FIRST DIMENSIONING PARAMETER FOR }
{              THE ARRAY .A( ).  ON ENTRY A( ) CONTAINS THE }
{              M BY N MATRIX .A.  ON EXIT A( ) CONTAINS THE }
{              PRODUCT MATRIX .Q*A.,WHERE Q IS AN M BY M }
{              ORTHOGONAL MATRIX GENERATEE IMPLICITLY BY }
{              THIS SUBROUTINE. }
{  B         - ON ENTRY B CONTAINS THE M-VECTOR B. }
{              ON EXIT B CONTAINS Q*B. }
{  X         - ON ENTRY X NEED NOT BE INITIALIZED. }
{              ON EXIT X WILL CONTAIN THE SOLUTION VECTOR. }
{  RNORM     - ON EXIT RNORM CONTAINS THE EUCLIEEAN NORM OF }
{              THE RESIEUAL VECTOR. }
{  W( )      - AN N-ARRAY OF WORKING SPACE.  ON EXIT W( ) }
{              WILL CONTAIN THE DUAL SOLUTION VECTOR.  W }
{              WILL SATISFY      W(I)=0. }
{              FOR ALL I IN SET P AND W(I) .LE. 0. }
{              FOR ALL I IN SET Z. }
{  ZZ( )     - AN M-ARRAY OF WORKING SPACE. }
{  INDEX( )  - AN INTEGER WORKING ARRAY OF LEANGTH AT }
{              LEAST N. }
{              ON EXIT THE CONTENTS OF THIS ARRAY DEFINE }
{              THE SETS  P AND  Z AS FOLLOWS... }
{                      INDEX(1) THRU INDEX(NSETP) = SET P. }
{                      INDEX(IZ1) THRU INDEX(IZ2) =SET Z. }
{                      IZ1 = NSETP + 1 = NPP1 }
{                      IZ2 = N . }
{  IERR      - THIS IS A DIAGNOSTICAL PARAMETER USED AS A }
{              SUCCESS-FAILURE FLAG WITH MEANINGS: }
{               0 - THE SOLUTION HAS BEEN COMPUTEE }
{                   SUCCESSFULLY. }
{              65 - THE DIMENSIONS OF THE PROBLEM ARE BAD. }
{              66 - ITERATION COUNT EXCEEEED  MORE THAN 3*N }
{                   ITERATIONS. }
{   EXTERNAL SUBROUTINES: }
{            ALN1R1 (HOUSEHOLEER TRANSFORMATION), }
{            ALN1R2 (COMPUTATION OF GIVENS ORTHOGONAL }
{                    ROTATION MATRIX), }
{            ALN1R3 (APPLICATION OF GIVENS MATRIX), }
{            ALN1R4 (COMPUTE DIFFERENCE DIFF=X-Y) }

{ ********************************************************* }

IERR := 0;
if ( M > 0 ) and ( N > 0 ) 
 then goto _10;
IERR := 65;
goto _450;
_10:
ITER := 0;
ITMAX := 3*N;

{     INITIALIZE THE ARRAYS  INDEX( ) AND X( ) . }

for I:=1 to N do
 begin
  X[I-1] := ZERO;
_20:
  INDEX[I-1] := I;
 end;
IZ2 := N;
IZ1 := 1;
NSETP := 0;
NPP1 := 1;

{ *************** MAIN LOOP BEGINS THERE **************** }

_30:

{    QUIT ALL COEFFICIENTS ARE ALREAEY IN THE SOLUTION. }
{    OR IF  M  COLS OF  A  HAVE BEEN TRIANGULARIZED. }

if ( IZ1 > IZ2 ) or ( NSETP >= M ) 
 then goto _350;

{    COMPUTE COMPONENTS OF THE DUAL (NEGATIVE GRADIENT) }
{    VECTOR W( ) . }

for IZ:=IZ1 to IZ2 do
 begin
  J := INDEX[IZ-1];
  SM := ZERO;
  for L:=NPP1 to M do
   begin
_40:
    SM := SM+A[(L-1)+(J-1)*MDA]*B[L-1];
   end;
_50:
  W[J-1] := SM;
 end;

{ ************* FIND LARGEST POSITIVE  W(J) . ********* }

_60:
WMAX := ZERO;
for IZ:=IZ1 to IZ2 do
 begin
  J := INDEX[IZ-1];
  if ( W[J-1] <= WMAX ) 
   then goto _70;
  WMAX := W[J-1];
  IZMAX := IZ;
_70:
 end;

{   IF  WMAX .LE. 0   GO TO TERMINATION . }
{   THIS INDICATES SATISFACTION OF THE KUHN-TUCKER }
{   CONDITIONS . }

if ( WMAX ) < 0
 then goto _350
 else if ( WMAX ) > 0
       then goto _80
       else goto _350;
_80:
IZ := IZMAX;
J := INDEX[IZ-1];

{   THE SIGN OF  W(J)  IS O.K. FOR J TO BE MOVEE TO SET P. }
{   BEGIN THE TRANSFORMATION AND CHECK NEW DIAGONAL ELEMENT }
{   TO AVOIE NEAR LINEAR DEPENDECE . }

ASAVE := A[(NPP1-1)+(J-1)*MDA];
ALN1R1(1,NPP1,NPP1+1,M,A[0+(J-1)*MDA],1,UP,DUMMY,1,1,0);
UNORM := ZERO;
if ( NSETP = 0 ) 
 then goto _100;
for L:=1 to NSETP do
 begin
_90:
  UNORM := UNORM+IntPower(A[(L-1)+(J-1)*MDA],2);
 end;
_100:
UNORM := Sqrt(UNORM);
if ( ALN1R4(UNORM+Abs(A[(NPP1-1)+(J-1)*MDA])*FACTOR,UNORM) ) < 0
 then goto _130
 else if ( ALN1R4(UNORM+Abs(A[(NPP1-1)+(J-1)*MDA])*FACTOR,UNORM) ) > 0
       then goto _110
       else goto _130;

{   COL J IS SUFFICIENTLY INDEPENEENT. COPY B INTO ZZ. }
{   UPDATE ZZ AND SOLVE FOR ZTEST ( -PROPOSEE NEW VALUE FOR }
{   X(J)) . }

_110:
for L:=1 to M do
 begin
_120:
  ZZ[L-1] := B[L-1];
 end;
ALN1R1(2,NPP1,NPP1+1,M,A[0+(J-1)*MDA],1,UP,ZZ,1,1,1);
ZTEST := ZZ[NPP1-1]/A[(NPP1-1)+(J-1)*MDA];

{ *************** SEE IF ZTEST IS POSITIVE ************ }

if ( ZTEST ) < 0
 then goto _130
 else if ( ZTEST ) > 0
       then goto _140
       else goto _130;

{   REJECT J AS A CANDIEATE TO BE MOVED FROM SET Z TO SET }
{   P. RESTORE A(NPP1,J). SET W(J)=0.  AND LOOP BACK TO }
{   TEST DUAL COEFFS AGAIN . }

_130:
A[(NPP1-1)+(J-1)*MDA] := ASAVE;
W[J-1] := ZERO;
goto _60;

{   THE INDEX J = INDEX(IZ) HAS BEEN SELECTEE TO BE MOVED }
{   FROM SET Z TO SET P. UPDATE B. UPDATE INDICES> APPLY }
{   HOUSEHOLEER TRANSFORMATIONS TO COLS IN NEW SET Z. }
{   ZERO SUBDIAGONAL ELTS IN COL J. SET W(J) = 0 . }

_140:
for L:=1 to M do
 begin
_150:
  B[L-1] := ZZ[L-1];
 end;
INDEX[IZ-1] := INDEX[IZ1-1];
INDEX[IZ1-1] := J;
IZ1 := IZ1+1;
NSETP := NPP1;
NPP1 := NPP1+1;
if ( IZ1 > IZ2 ) 
 then goto _170;
for JZ:=IZ1 to IZ2 do
 begin
  JJ := INDEX[JZ-1];
_160:
  ALN1R1(2,NSETP,NPP1,M,A[0+(J-1)*MDA],1,UP,A[0+(JJ-1)*MDA],1,MDA,1);
 end;
_170:
if ( NSETP = M ) 
 then goto _190;
for L:=NPP1 to M do
 begin
_180:
  A[(L-1)+(J-1)*MDA] := ZERO;
 end;
_190:
W[J-1] := ZERO;

{ ******  SOLVE THE TRIANGULAR SYSTEM.  ************ }
{   STORE THE SOLUTION TEMPORARILY IN ZZ( ) ******** }

NEXT := 200;
goto _400;
_200:

{ ************** SECONEARY LOOP BEGINS HERE  ************ }

{   ITERATION COUNTER . }

_210:
ITER := ITER+1;
if ( ITER <= ITMAX ) 
 then goto _220;
IERR := 66;
goto _350;
_220:

{   SEE IF ALL NEW CONSTRAINEE COEFFS ARE FEASIBLE. }
{   IF NOT COMPUTE ALPHA . }

ALPHA := TWO;
for IP:=1 to NSETP do
 begin
  L := INDEX[IP-1];
  if ( ZZ[IP-1] ) < 0
   then goto _230
   else if ( ZZ[IP-1] ) > 0
         then goto _240
         else goto _230;

_230:
  T := -X[L-1]/(ZZ[IP-1]-X[L-1]);
  if ( ALPHA <= T ) 
   then goto _240;
  ALPHA := T;
  JJ := IP;
_240:
 end;

{   IF ALL NEW CONSTRAINEE COEFFS ARE FEASIBLE THEN }
{   ALPHA WILL STILL = 2. IF SO EXIT FROM SECONEARY LOOP }
{   TO MAIN LOOP . }

if ( ALPHA = TWO ) 
 then goto _330;

{   OTHERWISE USE ALPHA WHICH WILL BE BETWEEN 0. AND 1. TO }
{   INTERPOLATE BETWEEN THE OLD X AND THE NEW ZZ . }

for IP:=1 to NSETP do
 begin
  L := INDEX[IP-1];
_250:
  X[L-1] := X[L-1]+ALPHA*(ZZ[IP-1]-X[L-1]);
 end;

{   MODIFY A AND B AND THE INDEX ARRAYS TO MOVE }
{   COEFFICIENT I  FROM SET  P  TO SET  Z . }

I := INDEX[JJ-1];
_260:
X[I-1] := ZERO;
if ( JJ = NSETP ) 
 then goto _290;
JJ := JJ+1;
for J:=JJ to NSETP do
 begin
  II := INDEX[J-1];
  INDEX[J-2] := II;
  ALN1R2(A[(J-2)+(II-1)*MDA],A[(J-1)+(II-1)*MDA],CC,SS,A[(J-2)+(II-1)*MDA]);
  A[(J-1)+(II-1)*MDA] := ZERO;
  for L:=1 to N do
   begin
    if ( L <> II ) 
     then ALN1R3(CC,SS,A[(J-2)+(L-1)*MDA],A[(J-1)+(L-1)*MDA]);
_270:
   end;
_280:
  ALN1R3(CC,SS,B[J-2],B[J-1]);
 end;
_290:
NPP1 := NSETP;
NSETP := NSETP-1;
IZ1 := IZ1-1;
INDEX[IZ1-1] := I;

{   SEE IF REMAINING COEFFS IN THE SET P ARE FEASIBLE. }
{   THEY SHOULE BE BECAUSE OF THE WAY ALPHA WAS DETERMINED. }
{   IF ANY ARE INFEASIBLE IT IS DUE TO ROUND-OFF ERRORS. }
{   ANY THAT ARE NONPOSITIVE WILL BE SET TO ZERO AND MOVEE }
{   FROM SET  P  TO SET  Z . }

for JJ:=1 to NSETP do
 begin
  I := INDEX[JJ-1];
  if ( X[I-1] ) < 0
   then goto _260
   else if ( X[I-1] ) > 0
         then goto _300
         else goto _260;
_300:
 end;

{   COPY  B( ) INTO ZZ( ). THEN SOLVE AGAIN AND LOOP BACK. }

for I:=1 to M do
 begin
_310:
  ZZ[I-1] := B[I-1];
 end;
NEXT := 320;
goto _400;
_320:
goto _210;

{ ********** END OF SECONEARY LOOP  ****************** }

_330:
for IP:=1 to NSETP do
 begin
  I := INDEX[IP-1];
_340:
  X[I-1] := ZZ[IP-1];
 end;

{   ALL NEW COEFFS ARE POSITIVE. LOOP BACK TO BEGINNING . }

goto _30;

{ ************** END OF MAIN LOOP  **************** }

{   COME TO HERE FOR TERMINATION . }
{   COMPUTE THE NORM OF THE FINAL RESIEUAL VECTOR . }

_350:
SM := ZERO;
if ( NPP1 > M ) 
 then goto _370;
for I:=NPP1 to M do
 begin
_360:
  SM := SM+IntPower(B[I-1],2);
 end;
goto _390;
_370:
for J:=1 to N do
 begin
_380:
  W[J-1] := ZERO;
 end;
_390:
RNORM := Sqrt(SM);
goto _450;

{   THE FOLLOWING BLOCK OF CODE IS USED AS INTERNAL }
{   SUBROUTINE TO SOLVE THE TRIANGULAR SYSTEM, PUTTING }
{   THE SOLUTION IN ZZ( ) . }

_400:
for L:=1 to NSETP do
 begin
  IP := NSETP+1-L;
  if ( L = 1 ) 
   then goto _420;
  for II:=1 to IP do
   begin
_410:
    ZZ[II-1] := ZZ[II-1]-A[(II-1)+(JJ-1)*MDA]*ZZ[IP+0];
   end;
_420:
  JJ := INDEX[IP-1];
_430:
  ZZ[IP-1] := ZZ[IP-1]/A[(IP-1)+(JJ-1)*MDA];
 end;
if ( NEXT=200 ) then goto _200 else
if ( NEXT=320 ) then goto _320;
_450:
end;

end.
